@@ -3,14 +3,16 @@ import json
 import logging
 from concurrent import futures
 
+import numpy as np
 import voluptuous as vol
 from aiohttp import web
 
 from ledfx.api import RestEndpoint
 from ledfx.events import Event
+from ledfx.utils import empty_queue
 
 _LOGGER = logging.getLogger(__name__)
-MAX_PENDING_MESSAGES = 1024
+MAX_PENDING_MESSAGES = 256
 
 BASE_MESSAGE_SCHEMA = vol.Schema(
     {
@@ -35,6 +37,10 @@ def websocket_handler(type):
     return function
 
 
+WEB_AUDIO_CLIENTS = set()
+ACTIVE_AUDIO_STREAM = None
+
+
 class WebsocketEndpoint(RestEndpoint):
 
     ENDPOINT_PATH = "/api/websocket"
@@ -71,6 +77,10 @@ class WebsocketConnection:
     def send(self, message):
         """Sends a message to the websocket connection"""
 
+        # If the queue is full, dump it and start again
+        if self._sender_queue.qsize() == MAX_PENDING_MESSAGES:
+            empty_queue(self._sender_queue)
+
         try:
             self._sender_queue.put_nowait(message)
         except asyncio.QueueFull:
@@ -107,7 +117,7 @@ class WebsocketConnection:
                 break
 
             try:
-                _LOGGER.debug("Sending websocket message")
+                # _LOGGER.debug("Sending websocket message")
                 await self._socket.send_json(message, dumps=json.dumps)
             except TypeError as err:
                 _LOGGER.error(
@@ -121,8 +131,42 @@ class WebsocketConnection:
     async def handle(self, request):
         """Handle the websocket connection"""
 
-        socket = self._socket = web.WebSocketResponse()
+        socket = self._socket = web.WebSocketResponse(
+            protocols=("http", "https", "ws", "wss")
+        )
+
+        # print(request.protocol)
+        # print(socket._protocols)
+        # headers = request.headers
+        # from aiohttp import hdrs
+        # protocol = None
+        # print(headers)
+        # print("SEC_WEBSOCKET_PROTOCOL", hdrs.SEC_WEBSOCKET_PROTOCOL)
+        # print(hdrs.SEC_WEBSOCKET_PROTOCOL in headers)
+        # if hdrs.SEC_WEBSOCKET_PROTOCOL in headers:
+        #     req_protocols = [
+        #         str(proto.strip())
+        #         for proto in headers[hdrs.SEC_WEBSOCKET_PROTOCOL].split(",")
+        #     ]
+        #     print("req",req_protocols)
+        #     for proto in req_protocols:
+        #         if proto in socket._protocols:
+        #             protocol = proto
+        #             break
+        #     else:
+        #         # No overlap found: Return no protocol as per spec
+        #         _LOGGER.warning(
+        #             "Client protocols %r don’t overlap server-known ones %r",
+        #             req_protocols,
+        #             socket._protocols,
+        #         )
+        # print(protocol)
+        # print(socket.can_prepare(request))
+        # print(socket._protocols)
+        # print(socket.ws_protocol)
+
         await socket.prepare(request)
+
         _LOGGER.info("Websocket connected.")
 
         self._receiver_task = asyncio.current_task(loop=self._ledfx.loop)
@@ -158,7 +202,7 @@ class WebsocketConnection:
             if socket.closed:
                 _LOGGER.info("Connection closed by client.")
             else:
-                _LOGGER.exception("Unexpected TypeError: {}".format(e))
+                _LOGGER.exception(f"Unexpected TypeError: {e}")
 
         except (asyncio.CancelledError, futures.CancelledError):
             _LOGGER.info("Connection cancelled")
@@ -188,7 +232,7 @@ class WebsocketConnection:
         def notify_websocket(event):
             self.send_event(message["id"], event)
 
-        _LOGGER.info(
+        _LOGGER.debug(
             "Websocket subscribing to event {} with filter {}".format(
                 message.get("event_type"), message.get("event_filter")
             )
@@ -202,12 +246,86 @@ class WebsocketConnection:
     @websocket_handler("unsubscribe_event")
     def unsubscribe_event_handler(self, message):
 
-        subscription_id = message["subscription_id"]
+        subscription_id = message["id"]
 
-        _LOGGER.info(
-            "Websocket unsubscribing from event {}".format(
-                message.get("event_type")
-            )
-        )
+        _LOGGER.debug(f"Websocket unsubscribing event id {subscription_id}")
         if subscription_id in self._listeners:
             self._listeners.pop(subscription_id)()
+
+    @websocket_handler("audio_stream_start")
+    def audio_stream_start_handler(self, message):
+        client = message.get("client")
+
+        if client in WEB_AUDIO_CLIENTS:
+            _LOGGER.warning(f"Web audio client {client} already exists")
+            return
+
+        _LOGGER.info(f"Web audio stream opened by client {client}")
+        WEB_AUDIO_CLIENTS.add(client)
+
+    @websocket_handler("audio_stream_stop")
+    def audio_stream_stop_handler(self, message):
+        client = message.get("client")
+        _LOGGER.info(f"Web audio stream closed by client {client}")
+        WEB_AUDIO_CLIENTS.discard(client)
+
+    @websocket_handler("audio_stream_config")
+    def audio_stream_config_handler(self, message):
+        _LOGGER.info(
+            "WebAudioConfig from {}: {}".format(
+                message.get("client"),
+                message.get("data"),
+            )
+        )
+
+    @websocket_handler("audio_stream_data")
+    def audio_stream_data_handler(self, message):
+
+        # _LOGGER.info(
+        #     "Websocket: {} incoming from {} with type {}".format(
+        #         message.get("event_type"),
+        #         message.get("client"),
+        #         type(message.get("data")),
+        #     )
+        # )
+
+        if not ACTIVE_AUDIO_STREAM:
+            return
+
+        client = message.get("client")
+
+        if ACTIVE_AUDIO_STREAM.client != client:
+            return
+        ACTIVE_AUDIO_STREAM.data = np.fromiter(
+            message.get("data").values(), dtype=np.float32
+        )
+
+
+class WebAudioStream:
+    def __init__(self, client: str, callback: callable):
+        self.client = client
+        self.callback = callback
+        self._data = None
+        self._active = False
+
+    def start(self):
+        self._active = True
+
+    def stop(self):
+        self._active = False
+
+    def close(self):
+        self._active = False
+
+    @property
+    def data(self, x):
+        return self._data
+
+    @data.setter
+    def data(self, x):
+        self._data = x
+        if self._active:
+            try:
+                self.callback(self._data, None, None, None)
+            except Exception as e:
+                _LOGGER.error(e)
