@@ -203,7 +203,11 @@ class QLC(Integration):
             await self._client.send(f"{int(widget_id)}|{value}")
 
     async def connect(self):
-        resolved_ip = resolve_destination(self._config["ip_address"])
+        resolved_ip = await resolve_destination(
+            self._ledfx.loop,
+            self._ledfx.thread_executor,
+            self._config["ip_address"],
+        )
         domain = f"{resolved_ip }:{self._config['port']}"
         url = f"http://{domain}/qlcplusWS"
         if self._client is None:
@@ -230,18 +234,20 @@ class QLC(Integration):
             self._connect_task = None
 
 
-class QLCWebsocketClient(aiohttp.ClientSession):
+class QLCWebsocketClient:
     def __init__(self, url, domain):
         super().__init__()
         self.websocket = None
         self.url = url
         self.domain = domain
+        self.session = aiohttp.ClientSession()
 
     async def connect(self):
         """Connect to the WebSocket."""
         while True:
             try:
-                self.websocket = await self.ws_connect(self.url)
+                self.websocket = await self.session.ws_connect(self.url)
+                # self.websocket = await self.ws_connect(self.url)
                 return True
             except aiohttp.client_exceptions.ClientConnectorError:
                 _LOGGER.info(
@@ -273,6 +279,7 @@ class QLCWebsocketClient(aiohttp.ClientSession):
             return
 
         await self.websocket.send_str(message)
+        # Every call to the logger is a performance hit
         _LOGGER.debug(f"Sent message {message} to {self.domain}")
 
     async def receive(self):
